/**********************************************************
* DIRECT3DAPP.CPP: A basic Direct3D Windows application.
**********************************************************/

#define INITGUID
#include <windows.h>
#include <windowsx.h>
#include <d3drm.h>
#include <d3drmwin.h>

/* Function prototypes */
LRESULT CALLBACK WndProc(HWND hWnd, UINT message,
    WPARAM wParam, LPARAM lParam);
void InitVariables();
BOOL InitDirect3D(HWND hWnd);
BOOL MakeScene();
BOOL CreateMesh();
BOOL CreateViewport();
BOOL CreateLight();

/* Global variables */
LPDIRECT3DRM pD3D;
LPDIRECT3DRMDEVICE pDevice;
LPDIRECT3DRMVIEWPORT pViewport;
LPDIRECT3DRMFRAME pParentFrame;
LPDIRECT3DRMFRAME pEye;
LPDIRECT3DRMMESHBUILDER pMeshBuilder;
LPDIRECTDRAWCLIPPER pClipper;

int WINAPI WinMain(HINSTANCE hCurrentInst,
    HINSTANCE hPrevInstance, PSTR lpszCmdLine,
    int nCmdShow)
{
    WNDCLASS wndClass;
    HWND hWnd;
    MSG msg;
	UINT width;
	UINT height;

    wndClass.style = CS_HREDRAW | CS_VREDRAW;
    wndClass.lpfnWndProc = WndProc;
    wndClass.cbClsExtra = 0;
    wndClass.cbWndExtra = 0;
    wndClass.hInstance = hCurrentInst;
    wndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
    wndClass.hbrBackground = GetStockObject(WHITE_BRUSH);
    wndClass.lpszMenuName = NULL;
    wndClass.lpszClassName = "D3DApp";

    RegisterClass(&wndClass);

    width = GetSystemMetrics(SM_CXSCREEN) / 2;
    height = GetSystemMetrics(SM_CYSCREEN) / 2;

    hWnd = CreateWindow(
      "D3DApp",             /* Window class's name.    */
      "Direct3D App",       /* Title bar text.         */
      WS_OVERLAPPEDWINDOW,  /* The window's style.     */
      10,                   /* X position.             */
      10,                   /* Y position.             */
      width,                /* Width.                  */
      height,               /* Height.                 */
      NULL,                 /* Parent window's handle. */
      NULL,                 /* Menu handle.            */
      hCurrentInst,         /* Instance handle.        */
      NULL);                /* No additional data.     */

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    while (GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message,
    WPARAM wParam, LPARAM lParam)
{
	HDC hDC;
	PAINTSTRUCT paintStruct;
	BOOL repaint;

    switch(message)
    {
	    case WM_CREATE:
			InitVariables();
			InitDirect3D(hWnd);
			return 0;

        case WM_PAINT:
        	repaint = GetUpdateRect(hWnd, NULL, FALSE);
	        if (!repaint)
		        return 0;

	        hDC = BeginPaint(hWnd, &paintStruct);

		    LPDIRECT3DRMWINDEVICE pWndDevice;
		    pDevice->QueryInterface(
			    IID_IDirect3DRMWinDevice,
			    (void**)&pWndDevice);
            pWndDevice->HandlePaint(hDC);
		    pWndDevice->Release();

        	EndPaint(hWnd, &paintStruct);

            return 0;

        case WM_TIMER:
    	    pD3D->Tick(D3DVALUE(1));
            return 0;

        case WM_DESTROY:
        	if (pMeshBuilder)
		        pMeshBuilder->Release();

	        if (pEye)
		        pEye->Release();

	        if (pViewport)
		        pViewport->Release();

	        if (pParentFrame)
		        pParentFrame->Release();

	        if (pDevice)
		        pDevice->Release();

	        if (pD3D)
		        pD3D->Release();

	        if (pClipper)
		        pClipper->Release();

            PostQuitMessage(0);
            return 0;
    }

    return DefWindowProc(hWnd, message, wParam, lParam);
}

void InitVariables()
{
	pD3D = NULL;
	pDevice = NULL;
	pViewport = NULL;
	pParentFrame = NULL;
	pEye = NULL;
	pMeshBuilder = NULL;
	pClipper = NULL;
}

BOOL InitDirect3D(HWND hWnd)
{
	HRESULT result = Direct3DRMCreate(&pD3D);
	if (result != D3DRM_OK)
		return FALSE;

	result = DirectDrawCreateClipper(0, &pClipper, 0);
	if (result != DD_OK)
		return FALSE;
	result = pClipper->SetHWnd(0, hWnd);
	if (result != DD_OK)
		return FALSE;

	RECT rect;
	GetClientRect(hWnd, &rect);
	result = pD3D->CreateDeviceFromClipper(pClipper,
		NULL, rect.right, rect.bottom, &pDevice);
	if (result != D3DRM_OK)
		return FALSE;
	pDevice->SetQuality(D3DRMRENDER_GOURAUD);

	pD3D->CreateFrame(0, &pParentFrame);
	if (result != D3DRM_OK)
		return FALSE;

	BOOL succeeded = MakeScene();
	if (!succeeded)
		return FALSE;

	SetTimer(hWnd, 1, 100, NULL);

	return TRUE;
}

BOOL MakeScene()
{
	BOOL succeeded = CreateMesh();
	if (!succeeded)
		return FALSE;

	succeeded = CreateViewport();
	if (!succeeded)
		return FALSE;

	succeeded = CreateLight();
	if (!succeeded)
		return FALSE;

	return TRUE;
}

BOOL CreateMesh()
{
	HRESULT result =
		pD3D->CreateMeshBuilder(&pMeshBuilder);
	if (result != D3DRM_OK)
		return FALSE;

	result = pMeshBuilder->Load("model.x",
		NULL, D3DRMLOAD_FROMFILE, NULL, NULL );
	if (result != D3DRM_OK)
	{
		MessageBox(0, "MODEL.X file missing. Please place\n\
a Direct3D X file with this name\nin the Direct3DApp folder.",
            "File Missing", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

	LPDIRECT3DRMFRAME pFrame;
	result =
		pD3D->CreateFrame(pParentFrame, &pFrame);
	if (result != D3DRM_OK)
		return FALSE;
	pFrame->AddVisual(pMeshBuilder);
	pFrame->SetRotation(pParentFrame, D3DVALUE(0.0),
		D3DVALUE(1.0), D3DVALUE(0.0), D3DVALUE(0.05));
	pFrame->Release();
	pFrame = NULL;

	return TRUE;
}

BOOL CreateViewport()
{
	HRESULT result = pD3D->
		CreateFrame(pParentFrame, &pEye);
	if (result != D3DRM_OK)
		return FALSE;
	pEye->SetPosition(pParentFrame, D3DVALUE(0),
		D3DVALUE(0), D3DVALUE(-40.0));
	result = pD3D->CreateViewport(pDevice, pEye,
		0, 0, pDevice->GetWidth(),
		pDevice->GetHeight(), &pViewport);
	if (result != D3DRM_OK)
		return FALSE;

	return TRUE;
}

BOOL CreateLight()
{
	LPDIRECT3DRMLIGHT pLight;
	HRESULT result = pD3D->
		CreateLightRGB(D3DRMLIGHT_DIRECTIONAL,
		D3DVALUE(1.0), D3DVALUE(1.0), D3DVALUE(1.0),
		&pLight);
	if (result != D3DRM_OK)
		return FALSE;

	LPDIRECT3DRMFRAME pLightFrame;
	result = pD3D->CreateFrame(pParentFrame,
		&pLightFrame );
	if (result != D3DRM_OK)
		return FALSE;

	pLightFrame->AddLight(pLight);
	pLightFrame->SetOrientation(pParentFrame,
		D3DVALUE(0.0), D3DVALUE(-1.0), D3DVALUE(1.0),
		D3DVALUE(0.0), D3DVALUE(1.0), D3DVALUE(0.0));
	pLight->Release();
	pLight = NULL;
	pLightFrame->Release();
	pLightFrame = NULL;

	return TRUE;
}

